import isAndroid from './detectors/isAndroid';
import isChrome from './detectors/isChrome';
import isEdge from './detectors/isEdge';
import isFirefox from './detectors/isFirefox';
import isIE from './detectors/isIE';
import isIE10 from './detectors/isIE10';
import isIE11 from './detectors/isIE11';
import isIOS from './detectors/isIOS';
import isIPad from './detectors/isIPad';
import isIPhone from './detectors/isIPhone';
import isIPod from './detectors/isIPod';
import isMac from './detectors/isMac';
import isOpera from './detectors/isOpera';
import isRetina from './detectors/isRetina';
import isSafari from './detectors/isSafari';
import isTouchScreen from './detectors/isTouchScreen';
import getViewport from './detectors/getViewport';
import isWin from './detectors/isWin';
var Environment = /** @class */ (function () {
    function Environment(params) {
        var _this = this;
        this.android = isAndroid();
        this.browser = !!window;
        this.chrome = isChrome();
        this.edge = isEdge();
        this.firefox = isFirefox();
        this.iPad = isIPad();
        this.iPhone = isIPhone();
        this.iPod = isIPod();
        this.ie = isIE();
        this.ie10 = isIE10();
        this.ie11 = isIE11();
        this.ios = isIOS();
        this.mac = isMac();
        this.opera = isOpera();
        this.pointerFocus = true;
        this.retina = isRetina();
        this.safari = isSafari();
        this.touchScreen = isTouchScreen();
        this.userHovering = !isTouchScreen();
        this.userTouching = isTouchScreen();
        this.utilityFocus = false;
        this.viewport = getViewport();
        this.win = isWin();
        this.onMouseMove = function () {
            // Mousemove fires on iOS on tap, right after touchstart and touchend
            _this.$mouseMoveCount++;
            if (!_this.userHovering && _this.$mouseMoveCount > 10) {
                _this.userTouching = false;
                _this.userHovering = true;
                _this.emitChange();
            }
        };
        this.onTouchStart = function () {
            if (!_this.userTouching) {
                _this.userHovering = false;
                _this.userTouching = true;
                _this.emitChange();
            }
            _this.$mouseMoveCount = 0;
        };
        this.onDocumentFocusIn = function (e) {
            if (_this.pointerFocus && (_this.ie10 || _this.ie11)) {
                if (e.originalEvent && (!e.originalEvent.fromElement && !e.originalEvent.relatedTarget)) {
                    _this.$isPointerFocus = true;
                    setTimeout(function () {
                        _this.$isPointerFocus = false;
                    });
                }
            }
            _this.pointerFocus = _this.$isPointerFocus;
            _this.utilityFocus = !_this.pointerFocus;
            _this.emitChange();
        };
        this.onDocumentKeydown = function () {
            clearTimeout(_this.$pointerFocusTimeout);
            _this.$isPointerFocus = false;
        };
        this.onDocumentClick = function (e) {
            if (!e.detail)
                return; // enter/space on focused element
            _this.$isPointerFocus = true;
            clearTimeout(_this.$pointerFocusTimeout);
            _this.$pointerFocusTimeout = window.setTimeout(function () {
                _this.$isPointerFocus = false;
            }, 600);
        };
        this.onWindowBlur = function () {
            var fix = function () {
                if (_this.pointerFocus) {
                    _this.$isPointerFocus = true;
                    setTimeout(function () {
                        _this.$isPointerFocus = false;
                    });
                }
                window.removeEventListener('focus', fix);
            };
            window.addEventListener('focus', fix);
        };
        this.onWindowResize = function () {
            _this.viewport = getViewport();
            _this.emitChange();
        };
        this.$listeners = [];
        this.$changeHandler = params ? params.onChange : null;
        this.$isPointerFocus = false;
        this.$mouseMoveCount = 0;
        this.$pointerFocusTimeout = 0;
        this.emitChange();
        window.addEventListener('touchstart', this.onTouchStart);
        window.addEventListener('mousemove', this.onMouseMove);
        window.addEventListener('blur', this.onWindowBlur);
        document.documentElement.addEventListener('focusin', this.onDocumentFocusIn);
        document.documentElement.addEventListener('keydown', this.onDocumentKeydown);
        document.documentElement.addEventListener('pointerdown', this.onDocumentClick, true);
        document.documentElement.addEventListener('pointerup', this.onDocumentClick, true);
        document.documentElement.addEventListener('mousedown', this.onDocumentClick, true);
        document.documentElement.addEventListener('mouseup', this.onDocumentClick, true);
        document.documentElement.addEventListener('click', this.onDocumentClick, true);
        window.addEventListener('orientationchange', this.onWindowResize);
        window.addEventListener('resize', this.onWindowResize);
    }
    Environment.prototype.destroy = function () {
        window.removeEventListener('touchstart', this.onTouchStart);
        window.removeEventListener('mousemove', this.onMouseMove);
        window.removeEventListener('blur', this.onWindowBlur);
        document.documentElement.removeEventListener('focusin', this.onDocumentFocusIn);
        document.documentElement.removeEventListener('keydown', this.onDocumentKeydown);
        document.documentElement.removeEventListener('pointerdown', this.onDocumentClick, true);
        document.documentElement.removeEventListener('pointerup', this.onDocumentClick, true);
        document.documentElement.removeEventListener('mousedown', this.onDocumentClick, true);
        document.documentElement.removeEventListener('mouseup', this.onDocumentClick, true);
        document.documentElement.removeEventListener('click', this.onDocumentClick, true);
        window.removeEventListener('orientationchange', this.onWindowResize);
        window.removeEventListener('resize', this.onWindowResize);
    };
    Environment.prototype.getState = function () {
        var _a = this, browser = _a.browser, viewport = _a.viewport, retina = _a.retina, touchScreen = _a.touchScreen, iPhone = _a.iPhone, iPad = _a.iPad, iPod = _a.iPod, android = _a.android, mac = _a.mac, win = _a.win, ios = _a.ios, opera = _a.opera, firefox = _a.firefox, safari = _a.safari, ie = _a.ie, ie10 = _a.ie10, ie11 = _a.ie11, edge = _a.edge, chrome = _a.chrome, userTouching = _a.userTouching, userHovering = _a.userHovering, utilityFocus = _a.utilityFocus, pointerFocus = _a.pointerFocus;
        return {
            browser: browser,
            viewport: viewport,
            retina: retina,
            touchScreen: touchScreen,
            iPhone: iPhone,
            iPad: iPad,
            iPod: iPod,
            android: android,
            mac: mac,
            win: win,
            ios: ios,
            opera: opera,
            firefox: firefox,
            safari: safari,
            ie: ie,
            ie10: ie10,
            ie11: ie11,
            edge: edge,
            chrome: chrome,
            userTouching: userTouching,
            userHovering: userHovering,
            utilityFocus: utilityFocus,
            pointerFocus: pointerFocus,
        };
    };
    Environment.prototype.subscribe = function (handler) {
        var _this = this;
        if (typeof handler !== 'function') {
            throw new TypeError('Handler must be function');
        }
        if (!this.$listeners.includes(handler)) {
            this.$listeners.push(handler);
        }
        return function () { return _this.unsubscribe(handler); };
    };
    Environment.prototype.unsubscribe = function (handler) {
        var index = this.$listeners.indexOf(handler);
        if (index > -1) {
            this.$listeners.splice(index, 1);
        }
    };
    Environment.prototype.emitChange = function () {
        var state = this.getState();
        if (typeof this.$changeHandler === 'function')
            this.$changeHandler(state);
        this.$listeners.forEach(function (fn) { return fn(state); });
    };
    return Environment;
}());
export default Environment;
